''' 
  initial comments
  
'''

#!/usr/bin/env python
from gimpfu import *
import decimal
import random
import time
import datetime
import os

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

# ========================================================
# ANCILLARY ROUTINES FOR PATTERN SELECTION
# --------------------------------------------------------

def StretchValues (layer, inMinabs, inMaxabs) :

    # VALUE 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < inMinabs) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_VALUE, fromV, toV)
        percent = percentile * 100.0  
        toV += 1
    #gimp.message ("percent VALUE at value 0 = " + str(percent))
    fromMinVALUE = toV - 1

    # =========== 
    # 255 QUOTE
    # ===========  
    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < inMaxabs) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_VALUE, fromV, toV)
        percent = percentile * 100.0  
        fromV -= 1
    #gimp.message ("percent VALUE at value 255 = " + str(percent))
    toMaxVALUE = fromV + 1
   
    #gimp.message ("Value From,To = "+str(fromMinVALUE)+","+str(toMaxVALUE))
    
    # =============================
    # APPLY MODIFICATIONS TO LEVELS
    # =============================
    
    pdb.gimp_levels (layer, 0, fromMinVALUE, toMaxVALUE, 1, 0, 255)
    
    return (layer)

############################################################################
#  M A I N   F U N C T I O N
# --------------------------------------------------------------------------

def WaterPastelPoster (inImage, inLayer, widthheight,
                       nrAreas, doFlatten) :
 
    pdb.gimp_context_push()
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    # set sample context
    pdb.gimp_context_set_sample_criterion (SELECT_CRITERION_COMPOSITE)
    pdb.gimp_context_set_sample_merged (False)
    pdb.gimp_context_set_sample_threshold (0.15)
    pdb.gimp_context_set_sample_threshold_int (15)
    pdb.gimp_context_set_sample_transparent (False)
    owidth = inImage.width
    oheight = inImage.height
    type = inImage.base_type
    name = inLayer.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # calculate scale factor for the new output image
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = realWidthHeight
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    # note : original will be scaled back after copy 
    pdb.gimp_image_scale (inImage, rwidth, rheight)
    
    if nrAreas == 0: # auto 
        nrAreas = int(widthheight/200) # fixed 5*1000  
    suffix = ", as a PastelPoster, on "+str(int(nrAreas))+" areas" 

# --------------------------------------------------------------------------------------
    
    newImage = pdb.gimp_image_new (rwidth, rheight, type)
    pdb.gimp_image_undo_freeze (newImage)                       # save time and space
    newBG = pdb.gimp_layer_new (newImage, rwidth, rheight, RGBA_IMAGE, "BG", 100.0, NORMAL_MODE)     # background layer
    newImage.add_layer (newBG, 0)
        
    pdb.gimp_edit_copy (inLayer)
    floatsel = pdb.gimp_edit_paste (newBG, True)
    pdb.gimp_floating_sel_anchor (floatsel)
    # check/avoid too bright source images
    (meanV,stddevV,b,c,d,e) = pdb.gimp_histogram (newBG, HISTOGRAM_VALUE, 0, 255)
    #info_msg ("meanV = "+str(meanV)+", stddevV = "+str(stddevV))
    if meanV < 64:
        modB = 30
    elif meanV > 192:
        modB = -30
    else:
        modB = 0
    if stddevV < 64:
        modC = 30
    elif stddevV > 192:
        modC = -30
    else:
        modC = 0
    pdb.gimp_brightness_contrast (newBG, modB, modC)
    (meanV,stddevV,b,c,d,e) = pdb.gimp_histogram (newBG, HISTOGRAM_VALUE, 0, 255)
    #info_msg ("newmeanV = "+str(meanV)+", newstddevV = "+str(stddevV))
    
    #pdb.gimp_display_new (newImage)
    #return
    
    newBG.mode = NORMAL_MODE
    newBG.name = "Background Image Layer"
    newBG.opacity = 100.0 
    pdb.gimp_layer_flatten (newBG)
    pdb.plug_in_sel_gauss (newImage, newBG, 20, 30)  # preliminary large simplification
    
    newBG2 = newBG.copy()
    newImage.add_layer (newBG2, 0)
    newBG2.mode = NORMAL_MODE
    newBG2.name = "Image Layer for Poster colours"
    newBG2.opacity = 100.0 
    pdb.gimp_layer_flatten (newBG2)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
#####################################################################################################
#   quantize the colour_areas using mode indexed on a duplicated image (INDEXED)
# =================================================================================================== 

    INDEXED_IMAGE = pdb.gimp_image_duplicate (newImage)
    INDEXED_DRAWABLE = INDEXED_IMAGE.active_layer
    INDEXED_DRAWABLE.name = "BG Layer"
    pdb.gimp_desaturate_full (INDEXED_DRAWABLE, DESATURATE_LUMINOSITY)
    
    #pdb.gimp_display_new (INDEXED_IMAGE)
    #return()
    
    # change image mode to indexed (this produces a cartoonized image, nrAreas colors)
    
    pdb.gimp_image_convert_indexed (INDEXED_IMAGE, 0, 0, nrAreas, 0, 0, "ignored")     
    is_indexed = pdb.gimp_drawable_is_indexed (INDEXED_DRAWABLE) 
    if is_indexed == 1:
        hidden_message = "OK, converted"
    else:    
        erro_msg ("ERROR! image not converted, exit")
        return
    #   
    # get the number of pixels in the colormap (which exists for indexed images)
    # ==========================================================================
    (count, colors) = pdb.gimp_image_get_colormap (INDEXED_IMAGE)
    # the count is nr_of_colors * 3, example 8*3=24
    nr_areas = (count/3) 
    #info_msg ("nr of colors in the palette: "+str(nr_areas)) 
  
    areas_palette = pdb.gimp_palette_new(pdb.gimp_image_get_name (INDEXED_IMAGE))
    
    index = 0
    while index < count:
    # Compute the color
        color = (colors[index], colors[index+1], colors[index+2])
    # Add to the filter temporary palette
        pdb.gimp_palette_add_entry (areas_palette, str(index/3), color)
        index+=3
        
    # Endwhile index<count
    # the temporary palette has been created            
    # ======================================

    # re-set image to RGB to proceed
    # ==============================
    pdb.gimp_image_convert_rgb (INDEXED_IMAGE)
    pdb.gimp_layer_add_alpha (INDEXED_DRAWABLE)                        #  RGBA
    
    #pdb.gimp_display_new (INDEXED_IMAGE)
    #return

#   ===================== 
#   back to primary image
#   ===================== 

    newINDEXED = pdb.gimp_layer_new_from_drawable (INDEXED_DRAWABLE, newImage)
    newImage.add_layer (newINDEXED, 0)
    newINDEXED.name = "Working Layer to hold Luminosity Areas"
    newINDEXED.opacity = 50.0
    newINDEXED.mode = SOFTLIGHT_MODE
    newINDEXED = StretchValues (newINDEXED, 0.1, 0.1)#   =====================
    
#   create work gradient
#   ===================== 
    
    (ng, gl) = pdb.gimp_gradients_get_list ("WaterPastelPoster")
    if ng == 1:
        pdb.gimp_gradient_delete ("WaterPastelPoster")   # delete if already existant
    #else:    
        #info_msg ("gradient does not exist yet, it will be created")
    WaterPastelPoster = pdb.gimp_gradient_new("WaterPastelPoster")   # (re-)create it
    
    pdb.gimp_gradient_segment_range_split_uniform(WaterPastelPoster, 0, 0, 7)
    
    pdb.gimp_gradient_segment_set_left_color(WaterPastelPoster, 0, (96,63,31), 100)
    pdb.gimp_gradient_segment_set_right_color(WaterPastelPoster, 0, (96,63,31), 100)
    pdb.gimp_gradient_segment_set_left_color(WaterPastelPoster, 1, (96,96,31), 100)
    pdb.gimp_gradient_segment_set_right_color(WaterPastelPoster, 1, (96,96,31), 100)
    pdb.gimp_gradient_segment_set_left_color(WaterPastelPoster, 2, (96,127,00), 100)
    pdb.gimp_gradient_segment_set_right_color(WaterPastelPoster, 2, (96,127,00), 100)
    pdb.gimp_gradient_segment_set_left_color(WaterPastelPoster, 3, (255,127,00), 100)
    pdb.gimp_gradient_segment_set_right_color(WaterPastelPoster, 3, (255,127,00), 100)
    pdb.gimp_gradient_segment_set_left_color(WaterPastelPoster, 4, (255,00,00), 100)
    pdb.gimp_gradient_segment_set_right_color(WaterPastelPoster, 4, (255,00,00), 100)
    pdb.gimp_gradient_segment_set_left_color(WaterPastelPoster, 5, (255,159,00), 100)
    pdb.gimp_gradient_segment_set_right_color(WaterPastelPoster, 5, (255,159,00), 100)
    pdb.gimp_gradient_segment_set_left_color(WaterPastelPoster, 6, (255,255,127), 100)
    pdb.gimp_gradient_segment_set_right_color(WaterPastelPoster, 6, (255,255,127), 100)
    
    pdb.gimp_gradient_segment_range_blend_colors(WaterPastelPoster, 0, 1)
    pdb.gimp_gradient_segment_range_blend_colors(WaterPastelPoster, 1, 2)
    pdb.gimp_gradient_segment_range_blend_colors(WaterPastelPoster, 2, 3)
    pdb.gimp_gradient_segment_range_blend_colors(WaterPastelPoster, 3, 4)
    pdb.gimp_gradient_segment_range_blend_colors(WaterPastelPoster, 4, 5)
    pdb.gimp_gradient_segment_range_blend_colors(WaterPastelPoster, 5, 6)

    pdb.gimp_gradients_refresh()
    pdb.gimp_context_set_gradient ("WaterPastelPoster")
    
    newCOLOURS = newINDEXED.copy()
    newImage.add_layer (newCOLOURS, 0)
    newCOLOURS.mode = HARDLIGHT_MODE
    newCOLOURS.name = "Image Layer for New colours"
    newCOLOURS.opacity = 50.0 
    
    pdb.plug_in_gradmap (newImage, newCOLOURS)
    
    newCOLOURS2 = newCOLOURS.copy()
    newImage.add_layer (newCOLOURS2, 0)
    pdb.plug_in_vinvert (newImage, newCOLOURS2)
    newCOLOURS2.mode = SOFTLIGHT_MODE
    newCOLOURS2.name = "2nd Image Layer for New colours"
    newCOLOURS2.opacity = 33.0
            
    pdb.gimp_palette_delete(areas_palette)
    pdb.gimp_selection_none (newImage)
    
    #pdb.gimp_display_new (newImage)
    #return
    
#================================================================================================ 
    
    # source on top
    pdb.gimp_image_raise_item_to_top (newImage, newBG)
    #pdb.gimp_hue_saturation (newBG, 0, 0, +25, +25)
    newBG.mode = HUE_MODE
    newBG.opacity = 35.0
    newTOP = newBG.copy()
    newImage.add_layer (newTOP, 0)
    newTOP.name = "2nd BG"
    newTOP.mode = VALUE_MODE
    newTOP.opacity = 70.0
    
    
    #pdb.gimp_display_new(newImage)
    #return()
    #
    if doFlatten == True:
        Outcome = pdb.gimp_image_flatten (newImage)
    # ----------------------------------------------------------------------------------------------- 
        Outcome.name = st+" "+name+suffix
    
    pdb.gimp_image_scale (inImage, owidth, oheight)                 # source back to original size
    pdb.gimp_displays_flush()
    pdb.gimp_image_undo_thaw (newImage)                       # close freeze
    pdb.gimp_display_new(newImage)

    pdb.gimp_context_pop()
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    if doFlatten == True:
        return (Outcome)
    else:
        return()

# =====================================================================================================

register(
    "WaterPastelPoster",    
    "to create an image looking as a poster with watercolour pastel colours",   
    "This script creates from the input drawable"+"\n a poster with watercolour pastel colours.",
    "Diego", 
    "Diego Nassetti ", 
    "September 2017",
    "WaterPastelPoster", 
    "RGB*,GRAY*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of Out Image"+
                              "\n (0=as input)", 2800, (0, 7000, 100)),
      (PF_ADJUSTMENT, "colorareas", "How many colour areas"+ 
                              "\n (14-42 recommended)", 28, (7, 70, 7)),
      (PF_TOGGLE, "flatten", "Flatten Outcome?", True),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    WaterPastelPoster,
    menu="<Image>/Diego/Artistic",
    )

main()